@@ -149,7 +149,7 @@ func parseUnprintableStyle(styleOption string) (textstyles.UnprintableStyleT, er
 	return 0, fmt.Errorf("Good ones are highlight or whitespace")
 }
 
-func parseScrollHint(scrollHint string) (twin.StyledRune, error) {
+func parseScrollHint(scrollHint string) (textstyles.CellWithMetadata, error) {
 	scrollHint = strings.ReplaceAll(scrollHint, "ESC", "\x1b")
 	hintAsLine := reader.NewLine(scrollHint)
 	parsedTokens := hintAsLine.HighlightedTokens(twin.StyleDefault, twin.StyleDefault, nil, nil, nil).StyledRunes
@@ -157,7 +157,7 @@ func parseScrollHint(scrollHint string) (twin.StyledRune, error) {
 		return parsedTokens[0], nil
 	}
 
-	return twin.StyledRune{}, fmt.Errorf("Expected exactly one (optionally highlighted) character. For example: 'ESC[2m…'")
+	return textstyles.CellWithMetadata{}, fmt.Errorf("Expected exactly one (optionally highlighted) character. For example: 'ESC[2m…'")
 }
 
 func parseShiftAmount(shiftAmount string) (uint, error) {
@@ -175,6 +175,21 @@ func parseShiftAmount(shiftAmount string) (uint, error) {
 	return uint(value), nil
 }
 
+func parseTabAmount(tabAmount string) (uint, error) {
+	value, err := strconv.ParseUint(tabAmount, 10, 32)
+	if err != nil {
+		return 0, err
+	}
+
+	if value < 1 {
+		return 0, fmt.Errorf("Tab size must be at least 1")
+	}
+
+	// Let's add an upper bound as well if / when requested
+
+	return uint(value), nil
+}
+
 func parseMouseMode(mouseMode string) (twin.MouseMode, error) {
 	switch mouseMode {
 	case "auto":
@@ -190,9 +205,27 @@ func parseMouseMode(mouseMode string) (twin.MouseMode, error) {
 
 func pumpToStdout(inputFilenames ...string) error {
 	if len(inputFilenames) > 0 {
+		stdinDone := false
+
 		// If we get both redirected stdin and an input filenames, should only
 		// copy the files and ignore stdin, because that's how less works.
 		for _, inputFilename := range inputFilenames {
+			if inputFilename == "-" && !term.IsTerminal(int(os.Stdin.Fd())) {
+				// "-" with redirected stdin means "read from stdin"
+				if stdinDone {
+					// stdin already drained, don't do it again
+					continue
+				}
+
+				_, err := io.Copy(os.Stdout, os.Stdin)
+				if err != nil {
+					return fmt.Errorf("Failed to copy stdin to stdout: %w", err)
+				}
+
+				stdinDone = true
+				continue
+			}
+
 			inputFile, _, err := reader.ZOpen(inputFilename)
 			if err != nil {
 				return fmt.Errorf("Failed to open %s: %w", inputFilename, err)
@@ -363,12 +396,13 @@ func pagerFromArgs(
 	unprintableStyle := flagSetFunc(flagSet, "render-unprintable", textstyles.UnprintableStyleHighlight,
 		"How unprintable characters are rendered: highlight or whitespace", parseUnprintableStyle)
 	scrollLeftHint := flagSetFunc(flagSet, "scroll-left-hint",
-		twin.NewStyledRune('<', twin.StyleDefault.WithAttr(twin.AttrReverse)),
+		textstyles.CellWithMetadata{Rune: '<', Style: twin.StyleDefault.WithAttr(twin.AttrReverse)},
 		"Shown when view can scroll left. One character with optional ANSI highlighting.", parseScrollHint)
 	scrollRightHint := flagSetFunc(flagSet, "scroll-right-hint",
-		twin.NewStyledRune('>', twin.StyleDefault.WithAttr(twin.AttrReverse)),
+		textstyles.CellWithMetadata{Rune: '>', Style: twin.StyleDefault.WithAttr(twin.AttrReverse)},
 		"Shown when view can scroll right. One character with optional ANSI highlighting.", parseScrollHint)
 	shift := flagSetFunc(flagSet, "shift", 16, "Horizontal scroll `amount` >=1, defaults to 16", parseShiftAmount)
+	tabSize := flagSetFunc(flagSet, "tab-size", 8, "Number of spaces per tab stop, defaults to 8", parseTabAmount)
 	mouseMode := flagSetFunc(
 		flagSet,
 		"mousemode",
@@ -435,15 +469,20 @@ func pagerFromArgs(
 	})
 
 	flagSetArgs := flagSet.Args()
-	if stdinIsRedirected && len(flagSetArgs) == 1 && flagSetArgs[0] == "-" {
-		// "-" is special, if we're getting data from stdin we should just ignore it
+	if stdinIsRedirected && len(flagSetArgs) == 0 {
+		// "-" is special if stdin is redirected, means "read from stdin"
 		//
 		// Ref: https://github.com/walles/moor/issues/162
-		flagSetArgs = []string{}
+		flagSetArgs = []string{"-"}
 	}
 
 	// Check that any input files can be opened
 	for _, inputFilename := range flagSetArgs {
+		if stdinIsRedirected && inputFilename == "-" {
+			// stdin can be opened
+			continue
+		}
+
 		// Need to check before newScreen() below, otherwise the screen
 		// will be cleared before we print the "No such file" error.
 		err := reader.TryOpen(inputFilename)
@@ -487,27 +526,44 @@ func pagerFromArgs(
 
 	var readerImpls []*reader.ReaderImpl
 	shouldFormat := *reFormat
-	if stdinIsRedirected {
-		// Display input pipe contents
-		readerImpl, err := reader.NewFromStream("", os.Stdin, formatter, reader.ReaderOptions{Lexer: *lexer, ShouldFormat: shouldFormat})
+	readerOptions := reader.ReaderOptions{Lexer: *lexer, ShouldFormat: shouldFormat}
 
-		// If the user is doing "sudo something | moor" we can't show the UI until
-		// we start getting data, otherwise we'll mess up sudo's password prompt.
-		readerImpl.AwaitFirstByte()
+	// MAN_PN is set by GNU man. Example value: "printf(1)"
+	stdinName := os.Getenv("MAN_PN")
+	if stdinName == "" {
+		stdinName = "<stdin>"
+	}
+
+	// Display the input file(s) contents
+	stdinDone := false
+	for _, inputFilename := range flagSetArgs {
+		var readerImpl *reader.ReaderImpl
+		var err error
+
+		if stdinIsRedirected && inputFilename == "-" {
+			if stdinDone {
+				// stdin already drained, don't do it again
+				continue
+			}
+
+			readerImpl, err = reader.NewFromStream(stdinName, os.Stdin, formatter, readerOptions)
+			if err != nil {
+				return nil, nil, chroma.Style{}, nil, logsRequested, err
+			}
+
+			// If the user is doing "sudo something | moor" we can't show the UI until
+			// we start getting data, otherwise we'll mess up sudo's password prompt.
+			readerImpl.AwaitFirstByte()
+
+			stdinDone = true
+		} else {
+			readerImpl, err = reader.NewFromFilename(inputFilename, formatter, readerOptions)
+		}
 
 		if err != nil {
 			return nil, nil, chroma.Style{}, nil, logsRequested, err
 		}
 		readerImpls = append(readerImpls, readerImpl)
-	} else {
-		// Display the input file(s) contents
-		for _, inputFilename := range flagSetArgs {
-			readerImpl, err := reader.NewFromFilename(inputFilename, formatter, reader.ReaderOptions{Lexer: *lexer, ShouldFormat: shouldFormat})
-			if err != nil {
-				return nil, nil, chroma.Style{}, nil, logsRequested, err
-			}
-			readerImpls = append(readerImpls, readerImpl)
-		}
 	}
 
 	// We got the first byte, this means sudo is done (if it was used) and we
@@ -548,6 +604,7 @@ func pagerFromArgs(
 	pager.ScrollLeftHint = *scrollLeftHint
 	pager.ScrollRightHint = *scrollRightHint
 	pager.SideScrollAmount = int(*shift)
+	pager.TabSize = int(*tabSize)
 
 	pager.TargetLine = targetLine
 	if *follow && pager.TargetLine == nil {
