@@ -74,9 +74,12 @@ type Pager struct {
 	isShowingHelp bool
 	preHelpState  *_PreHelpState
 
-	// NewPager shows lines by default, this field can hide them
+	// User preference
 	ShowLineNumbers bool
 
+	// Current state, initialized in StartPaging()
+	showLineNumbers bool
+
 	StatusBarStyle StatusBarOption
 	ShowStatusBar  bool
 
@@ -88,10 +91,12 @@ type Pager struct {
 	QuitIfOneScreen bool
 
 	// Ref: https://github.com/walles/moor/issues/94
-	ScrollLeftHint  twin.StyledRune
-	ScrollRightHint twin.StyledRune
+	ScrollLeftHint  textstyles.CellWithMetadata
+	ScrollRightHint textstyles.CellWithMetadata
 
-	SideScrollAmount int // Should be positive
+	SideScrollAmount int // Left / right arrow keys scroll amount
+
+	TabSize int // Number of spaces per tab, default 8, should be positive
 
 	// If non-nil, scroll to this line as soon as possible. Set this value to
 	// IndexMax() to follow the end of the input (tail).
@@ -210,12 +215,14 @@ func NewPager(readers ...*reader.ReaderImpl) *Pager {
 		currentReader:    0,
 		readerSwitched:   make(chan struct{}, 1),
 		quit:             false,
-		ShowLineNumbers:  true,
+		ShowLineNumbers:  true, // Constant throghout the lifetime of the pager
+		showLineNumbers:  true, // Will be updated over time
 		ShowStatusBar:    true,
 		DeInit:           true,
 		SideScrollAmount: 16,
-		ScrollLeftHint:   twin.NewStyledRune('<', twin.StyleDefault.WithAttr(twin.AttrReverse)),
-		ScrollRightHint:  twin.NewStyledRune('>', twin.StyleDefault.WithAttr(twin.AttrReverse)),
+		TabSize:          8, // This is what less defaults to
+		ScrollLeftHint:   textstyles.CellWithMetadata{Rune: '<', Style: twin.StyleDefault.WithAttr(twin.AttrReverse)},
+		ScrollRightHint:  textstyles.CellWithMetadata{Rune: '>', Style: twin.StyleDefault.WithAttr(twin.AttrReverse)},
 		scrollPosition:   newScrollPosition(name),
 	}
 
@@ -238,11 +245,11 @@ func (p *Pager) visibleHeight() int {
 	return height
 }
 
-// How many cells are needed for this line number?
+// How many cells are needed for this line number? Includes padding.
 //
 // Returns 0 if line numbers are disabled.
 func (p *Pager) getLineNumberPrefixLength(lineNumber linemetadata.Number) int {
-	if !p.ShowLineNumbers {
+	if !p.showLineNumbers {
 		return 0
 	}
 
@@ -318,13 +325,13 @@ func (p *Pager) Quit() {
 
 // Negative deltas move left instead
 func (p *Pager) moveRight(delta int) {
-	if p.ShowLineNumbers && delta > 0 {
-		p.ShowLineNumbers = false
+	if p.showLineNumbers && delta > 0 {
+		p.showLineNumbers = false
 		return
 	}
 
 	if p.leftColumnZeroBased == 0 && delta < 0 {
-		p.ShowLineNumbers = true
+		p.showLineNumbers = true
 		return
 	}
 
@@ -405,7 +412,14 @@ func (p *Pager) StartPaging(screen twin.Screen, chromaStyle *chroma.Style, chrom
 		}
 	}()
 
+	p.showLineNumbers = p.ShowLineNumbers
+
 	textstyles.UnprintableStyle = p.UnprintableStyle
+	if p.TabSize > 0 {
+		// "0" = unset, stay at the default. If the tab size is negative, just
+		// ignoring it seems like the right move.
+		textstyles.TabSize = p.TabSize
+	}
 	consumeLessTermcapEnvs(screen.TerminalBackground(), chromaStyle, chromaFormatter)
 	styleUI(screen.TerminalBackground(), chromaStyle, chromaFormatter, p.StatusBarStyle, p.WithTerminalFg)
 
@@ -506,11 +520,10 @@ func (p *Pager) StartPaging(screen twin.Screen, chromaStyle *chroma.Style, chrom
 			// Also, we only do this if we have exactly one reader, because
 			// that's what less does.
 			if len(p.readers) == 1 && p.QuitIfOneScreen && !p.isShowingHelp && r.Done.Load() && r.HighlightingDone.Load() {
-				width, height := p.screen.Size()
-				if fitsOnOneScreen(r, width, height-p.DeInitFalseMargin) {
+				if p.fitsOnOneScreen() {
 					// Ref:
 					// https://github.com/walles/moor/issues/113#issuecomment-1368294132
-					p.ShowLineNumbers = false // Requires a redraw to take effect, see below
+					p.showLineNumbers = false // Requires a redraw to take effect, see below
 					p.DeInit = false
 					p.quit = true
 
@@ -589,7 +602,65 @@ func (p *Pager) StartPaging(screen twin.Screen, chromaStyle *chroma.Style, chrom
 // shell prompt.
 //
 // This way nothing gets scrolled off screen after we exit.
-func fitsOnOneScreen(reader *reader.ReaderImpl, width int, height int) bool {
+func (p *Pager) fitsOnOneScreenWrapped() bool {
+	if len(p.readers) != 1 {
+		// At most one screen will fit on one screen...
+		return false
+	}
+
+	// Create a fake screen of height + 1 lines
+	width, height := p.screen.Size()
+
+	// If the screen height is one, and the prompt height is zero, then the last
+	// line number will be zero. But since we want one extra line to check for
+	// overflow, we now want the last line number to be one.
+	//
+	// So if the initial height is 1, we want the last line number to be 1.
+	// Which is what we get from here.
+	lastScreenRow := height - p.DeInitFalseMargin
+
+	// If the last screen row is supposed to be one, we need to set the height
+	// to two. So we add one here.
+	testScreenHeight := lastScreenRow + 1
+	testScreen := twin.NewFakeScreen(width, testScreenHeight)
+
+	// Create a fake pager for that screen, with no status bar, and matching
+	// line number settings + tab size and wrap settings
+	p.readerLock.Lock()
+	fakePager := NewPager(p.readers[0])
+	p.readerLock.Unlock()
+	fakePager.screen = testScreen
+
+	// If we drop out because of quit-if-one-screen, we will not print any line numbers
+	fakePager.showLineNumbers = false
+
+	fakePager.WrapLongLines = p.WrapLongLines
+	fakePager.ShowStatusBar = false // We are only interested in content lines
+	fakePager.TabSize = p.TabSize
+
+	// Render on our test screen
+	rendered := fakePager.renderLines()
+
+	return len(rendered.lines) < testScreenHeight
+}
+
+func (p *Pager) fitsOnOneScreen() bool {
+	if len(p.readers) != 1 {
+		// At most one screen will fit on one screen...
+		return false
+	}
+
+	if p.WrapLongLines {
+		return p.fitsOnOneScreenWrapped()
+	}
+
+	width, height := p.screen.Size()
+	height -= p.DeInitFalseMargin
+
+	p.readerLock.Lock()
+	reader := p.readers[0]
+	p.readerLock.Unlock()
+
 	if reader.GetLineCount() > height {
 		return false
 	}
@@ -610,8 +681,8 @@ func fitsOnOneScreen(reader *reader.ReaderImpl, width int, height int) bool {
 // "leaving" pager contents on screen after exit.
 func (p *Pager) ReprintAfterExit() error {
 	// Figure out how many screen lines are used by pager contents
-	renderedScreenLines, _ := p.renderScreenLines()
-	screenLinesCount := len(renderedScreenLines)
+	renderedScreen := p.renderLines()
+	screenLinesCount := len(renderedScreen.lines)
 
 	_, screenHeight := p.screen.Size()
 	screenHeightWithoutFooter := screenHeight - p.DeInitFalseMargin
